/*
 *  /MathJax-v2/localization/ca/FontWarnings.js
 *
 *  Copyright (c) 2009-2018 The MathJax Consortium
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

MathJax.Localization.addTranslation("ca","FontWarnings",{version:"2.7.9",isLoaded:true,strings:{webFont:"MathJax est\u00E0 utilitzant fonts basades en web per mostrar les expressions matem\u00E0tiques d'aquesta p\u00E0gina. Aix\u00F2 porta un temps per descarregar, de tal manera que la p\u00E0gina es mostrar\u00E0 m\u00E9s r\u00E0pidament si instal\u00B7leu les fonts matem\u00E0tiques directament a la vostra carpeta de sistema de fonts.",imageFonts:"MathJaz est\u00E0 utilitzant les seves fonts d'imatge en comptes de fonts locals o basades en web. Aix\u00F2 pot fer que la composici\u00F3 de la p\u00E0gina sigui m\u00E9s lenta que l'habitual, i les expressions matem\u00E0tiques poden no imprimir-se a resoluci\u00F3 completa en la vostra impressora.",noFonts:"MathJax no pot localitzar una font per a utilitzar-la a l'hora de mostrar les expressions matem\u00E0tiques, de tal manera que es recolza en car\u00E0cters Unicode gen\u00E8rics, amb el sup\u00F2sit que el vostre navegador ser\u00E0 capa\u00E7 de mostrar-les. Alguns car\u00E0cters poden no mostrar-se correctament, o fins i tot poden no mostrar-se en absolut.",webFonts:"La majoria de navegadors moderns permeten la desc\u00E0rrega de les fonts. Si actualitzeu a una versi\u00F3 m\u00E9s nova del vostre navegador (o si canvieu de navegador) podreu millorar la qualitat de les expressions matem\u00E0tiques d'aquesta p\u00E0gina.",fonts:"MathJax pot utilitzar tant les [fonts STIX](%1) com les [fonts TeX MathJax](%2). Descarregueu i instal\u00B7leu una d'aquestes fonts per millorar la vostra experi\u00E8ncia MathJax.",STIXPage:"Aquesta p\u00E0gina est\u00E0 dissenyada per usar les [fonts STIX](%1). Descarregueu i instal\u00B7leu aquestes fonts per millorar la vostra experi\u00E8ncia MathJax.",TeXPage:"Aquesta p\u00E0gina est\u00E0 dissenyada per usar les [fonts TeX MathJax](%1). Descarregueu i instal\u00B7leu aquestes fonts per millorar la vostra experi\u00E8ncia MathJax."}});MathJax.Ajax.loadComplete("[MathJax]/localization/ca/FontWarnings.js");
